"""
Authors: Peter Mawhorter
Consulted:
Date: 2022-04-09
Purpose: Testing code for the payroll task involving reading + writing
  CSV files.
"""

import optimism
import payroll

import os

optimism.skipChecksAfterFail('all')
# If you want to keep running checks after one fails like optimism used
# to do, uncomment this line:
# optimism.skipChecksAfterFail('case')

# Check for extra or missing reports files...
try:
    reportFiles = os.listdir('reports')
except (OSError, PermissionError, FileNotFoundError):
    print(
        "Could not access the 'reports' directory."
        "\nMake sure that you've got all of the starter files together."
    )
    print("Cancelling tests.")
    exit(1)

expected = {
    'Hamingfram.csv',
    'Hamneed.csv',
    'Newt.csv',
    'Notuck.csv',
    'Thamwal.csv',
    'Whyland.csv',
    'Wolleslu.csv',
}
found = set(reportFiles)
missing = expected - found
extra = found - expected
extraCSV = [f for f in extra if f.endswith(".csv")]

if missing:
    missing = '\n'.join(missing)
    print(f"The following report files are missing:\n{missing}")
    print("Make sure that you have all of the starter files.")
    print("Cancelling tests.")
    exit(1)

if extraCSV:
    extraCSV = '\n'.join(extraCSV)
    print(f"You have extra files in your 'reports' directory:\n{extraCSV}")
    print("You must delete these extra files before testing.")
    print("Cancelling tests.")
    exit(1)


# Test cases for columnAverage
testCA = optimism.testFunctionMaybe(payroll, "columnAverage")
testCA.case('reports/Wolleslu.csv', 'Hours').checkReturnValue(
    134.30769230769232
)
testCA.case('reports/Wolleslu.csv', 'Pay').checkReturnValue(
    2070.4869230769227
)
testCA.case('reports/Wolleslu.csv', 'Wage').checkReturnValue(
    13.280833333333334
)
testCA.case('reports/Newt.csv', 'Overtime').checkReturnValue(0)
testCA.case('reports/Newt.csv', 'Name').checkReturnValue(None)
testCA.case('special/Offset.csv', 'Wage').checkReturnValue(12.344)
testCA.case('special/Nowhere.csv', 'Wage').checkReturnValue(None)


# Test cases for mergeReports
testMR = optimism.testFunctionMaybe(payroll, "mergeReports")
testMR.case(
    'reports/Wolleslu.csv',
    'reports/Hamingfram.csv',
    'results/Wollesfram.csv'
).checkFileLines(
    'results/Wollesfram.csv',
    'Name,Id,Hours,Overtime,Wage,Pay,Benefits',
    'Walter,379325,140,0,14.7,2057.66,617.3',
    'Jolynn,379327,132,0,11.58,1528.22,458.46',
    'Anastasia,379330,135,0,13.7,1849.1,554.73',
    'John,379333,132,0,11.6,1530.63,459.19',
    'Emberlynn,379334,156,0,15.06,2349.61,704.88',
    'Mahdi,379337,97,0,11.33,1099.13,329.74',
    'Lazarus,379339,141,0,14.32,2018.98,605.69',
    'Yonatan,379341,101,0,11.56,1167.64,350.29',
    'Colt,379344,148,0,14.27,2112.16,633.65',
    'Levi,379345,140,0,11.71,1639.92,491.98',
    'Ryder,379346,142,0,14.58,2070.51,621.15',
    'Charlie,379347,148,0,14.96,2214.11,664.23',
    'Genesis,379315,134,0,salaried,5278.66,2903.26',
    'Eleanor,379351,133,0,12.17,1618.41,485.52',
    'Abigail,379352,146,0,11.41,1666.24,499.87',
    'Mila,379361,137,0,14.09,1930.0,579.0',
    'Ethan,379362,76,0,13.99,1063.35,53.17',
    'Lily,379363,66,0,11.65,768.71,38.44',
    'Sadie,379364,125,0,12.31,1538.81,461.64',
    'Gabriel,379350,160,57,salaried,5691.47,3130.31'
)
testMR.case(
    'reports/Newt.csv',
    'special/Mixedup.csv',
    'results/MixedNewt.csv'
).checkFileLines(
    'results/MixedNewt.csv',
    'Name,Id,Hours,Overtime,Wage,Pay,Benefits',
    'Skye,379243,109,0,11.39,1241.93,372.58',
    'Jay,379245,76,0,13.85,1052.89,52.64',
    'Naliyah,379247,186,0,10.34,1923.65,769.46',
    'Ibrahim,379248,145,0,14.3,2073.74,622.12',
    'Londyn,379249,207,0,12.53,2592.75,1037.1',
    'Arlo,379251,145,0,13.91,2016.96,605.09',
    'Jordan,379240,103,0,salaried,5066.93,2786.81',
    'Luna,379328,116,0,11.62,1348.22,404.47',
    'Phoenix,379330,88,0,13.33,1172.62,351.79',
    'Khiara,379331,125,0,10.91,1364.11,409.23',
    'Helene,379333,82,0,13.1,1073.85,322.16',
    'Alexander,379334,93,0,12.76,1186.57,355.97',
    'Hadley,379327,160,76,salaried,5683.96,3126.18'
)


# Test cases for updateReport
testUR = optimism.testFunctionMaybe(payroll, "updateReport")
testUR.case(
    'reports/Newt.csv',
    'updates/Newt2.csv',
    'results/NewtUpdated.csv'
).checkFileLines(
    'results/NewtUpdated.csv',
    'Name,Id,Hours,Overtime,Wage,Pay,Benefits',
    'Jay,379245,76,0,13.85,1052.89,52.64',
    'Naliyah,379247,186,0,10.34,1923.65,769.46',
    'Ibrahim,379248,145,0,14.3,2073.74,622.12',
    'Londyn,379249,207,0,12.53,2592.75,1037.1',
    'Skye,379243,113,0,11.39,1287.49,383.62',
    'Arlo,379251,137,0,13.91,1919.59,575.92',
    'Jordan Jr.,379267,25,0,25.00,525.00,0.00',
    'Jordan,379240,128,0,salaried,5066.93,2786.81'
)

testUR.case(
    'reports/Thamwal.csv',
    'updates/Thamwal2.csv',
    'results/ThamwallUpdated.csv'
).checkFileLines(
    'results/ThamwallUpdated.csv',
    'Name,Id,Hours,Overtime,Wage,Pay,Benefits',
    'Penelope,379278,138,0,13.14,1812.86,543.86',
    'Dasia,379279,71,0,12.14,861.76,43.09',
    'Leo,379280,61,0,11.12,678.02,33.9',
    'Nicholas,379281,139,0,11.5,1598.71,479.61',
    'Remi,379292,54,0,10.48,566.17,28.31',
    'Lynnea,379295,147,0,12.67,1862.27,558.68',
    'Xavier,379297,150,0,13.82,2072.94,621.88',
    'Tucker,379300,148,0,12.3,1819.77,545.93',
    'Zamir,379304,59,0,11.93,703.63,35.18',
    'Isaiah,379305,139,0,12.75,1772.46,531.74',
    'Mason,379277,160,38,salaried,5676.21,3121.92',
    'Luis,379291,0,0,14.53,0,0',
    'Luisa,379291,57,0,14.53,828.04,41.4'
)

testUR.case(
    'reports/Whyland.csv',
    'updates/Whyland2.csv',
    'results/WhylandUpdated.csv'
).checkFileLines(
    'results/WhylandUpdated.csv',
    'Name,Id,Hours,Overtime,Wage,Pay,Benefits',
    'Patricio,379367,70,0,13.54,947.66,47.38',
    'Charlotte,379369,149,0,11.72,1746.33,523.9',
    'Paxson,379373,165,0,12.55,2070.44,828.18',
    'Ava,379374,133,0,11.42,1518.52,455.56',
    'Julianne,379375,133,0,11.75,1562.23,468.67',
    'Isaiah,379376,141,0,14.46,2039.44,611.83',
    'Dagny,379380,71,0,13.56,962.5,48.13',
    'Omega,379383,156,0,14.94,2330.17,699.05',
    'Griffin,379386,162,0,10.5,1701.66,680.66',
    'Carlos,379387,69,0,13.65,942.07,47.1',
    'Merrin,379389,66,0,13.61,898.14,44.91',
    'Ayden,379366,109,0,salaried,5668.94,3117.92',
    'Fenix,379482,85,0,salaried,6000.00,3500.00',
    'Ava,379275,160,36,salaried,5739.69,3156.83'
)


# Tests for salariesReport
testSR = optimism.testFunctionMaybe(payroll, "salariesReport")
testSR.case(
    'reports/Wolleslu.csv',
    'results/salaries_Wolleslu.csv'
).checkFileLines(
    'results/salaries_Wolleslu.csv',
    'Name,Id,Pay,Hours',
    'Genesis,379315,5278.66,134'
)

testSR.case(
    'reports/Whyland.csv',
    'results/salaries_Whyland.csv'
).checkFileLines(
    'results/salaries_Whyland.csv',
    'Name,Id,Pay,Hours',
    'Ayden,379366,5668.94,109',
    'Fenix,379482,6000.00,85'
)
